/*
 * Copyright 2011 Google Inc.
 * Copyright 2014 Andreas Schildbach
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.twostack.bitcoin4j.base58;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.twostack.bitcoin4j.address.Base58;

import java.math.BigInteger;
import java.util.Arrays;
import java.util.Collection;

import static org.junit.Assert.assertEquals;
import static org.junit.runners.Parameterized.Parameters;

@RunWith(Parameterized.class)
public class Base58EncodeTest {

    private byte[] input;
    private String expected;

    public Base58EncodeTest(byte[] input, String expected) {
        this.input = input;
        this.expected = expected;
    }

    @Parameters
    public static Collection<Object[]> parameters() {
        return Arrays.asList(new Object[][]{
                {"Hello World".getBytes(), "JxF12TrwUP45BMd"},
                {BigInteger.valueOf(3471844090L).toByteArray(), "16Ho7Hs"},
                {new byte[1], "1"},
                {new byte[7], "1111111"},
                {new byte[0], ""}
        });
    }

    @Test
    public void testEncode() {
        assertEquals(expected, Base58.encode(input));
    }
}
